%anamorphic
% Loads a bitmap image and fits this within a unit circle. An
% anamorphic projection is then determined such that reflection of the
% projection in a vertically oriented cylindrical mirror has the same
% proportions as the original 2D image, but appears to float in space.
% 
% LAST UPDATED by Andy French Dec 2021.

function anamorphic

%Set opening angle of anamorphic arc /degrees
arc_deg = 160;

%Radial scaling factor
Rf = 3;

%Plot image in circle?
plot_image = 1;

%Plot grid?
plot_grid = 1;

%

%Import bitmap image of object and store RGB values in range 0...1 as a
%Iwidth x Iheight x 3 array
[filename,pathname] = uigetfile('*.*','Select bitmap to be anamorphed!');
if isempty(filename); return; end
I = imread([pathname,'\',filename]); I = double( flipdim(I,1) )/255;

%Get width and height in pixels
dim = size(I); Iwidth = dim(2); Iheight = dim(1);

%Scale image so it fits into a unit circle centred on the origin
sf = 0.5*sqrt( Iwidth^2 + Iheight^2 );

%Determine x,y coordinates of pixel locations
x = linspace(0,Iwidth/sf,Iwidth) - 0.5*Iwidth/sf;
y = linspace(0,Iheight/sf,Iheight) - 0.5*Iheight/sf ;
[x,y] = meshgrid(x,y);

%Create figure and plot a circle of radius R. If option set, plot original
%image within the circle. This is the untransformed original.
figure('name','anamorphic');
if plot_image==1; surf(x,y,-ones(Iheight,Iwidth),I); end
shading interp; view(2); hold on; axis equal; axis off;
theta = linspace( 0, 2*pi, 1000 );
plot( cos(theta), sin(theta), 'k-','linewidth',3 );

%Determine x,y coordinates of anamorphic grid, i.e. the pixel locations for
%the transformed image.
r = linspace( 1, Rf*Iheight/sf + 1, Iheight );
theta = linspace( -0.5*(arc_deg*pi/180), 0.5*(arc_deg*pi/180), Iwidth );
[theta,r] = meshgrid(theta,r);
x = r.*sin(theta); y = -0.5*Iheight/sf -r.*cos(theta);

%Plot anamorphic image
surf( x,y,-ones(Iheight,Iwidth),I); shading interp;

%Add grid?
if plot_grid==1
    xlims = get(gca,'xlim'); ylims = get(gca,'ylim');
    xlims = [ floor(xlims(1)), ceil(xlims(2)) ];
    ylims = [ floor(ylims(1)), ceil(ylims(2)) ];
    set(gca,'visible','on','gridlines','-','xtick',xlims(1):xlims(end),...
        'ytick',ylims(1):ylims(end),'xlim',xlims,'ylim',ylims );
    box on;
    %Plot centre of anamorphic sector centre
    plot( 0, -0.5*Iheight/sf, 'r*' );
end
    
%Save 300DPI PNG image and close figure
if plot_image==1
    filename = [pathname,'\',filename(1:end-4),'_anamorph_with_image.png'];
else
    filename = [pathname,'\',filename(1:end-4),'_anamorph.png'];
end
print( gcf, filename,'-dpng','-r300' ); close(gcf);

%End of code